package storage

import (
	"database/sql"
	"fmt"
	"strconv"
	"strings"

	"github.com/google/uuid"
	_ "github.com/lib/pq"
	"github.com/rs/zerolog/log"
	"github.com/spf13/viper"
)

type PostgresqlStorage struct {
	DbConnection *sql.DB
	Schema       string
	Tables       map[string]string
}

func NewPostgresqlStorage(cfg *viper.Viper) (PostgresqlStorage, error) {
	var (
		host                  = cfg.GetString("storage.db.psql.host")
		port                  = cfg.GetString("storage.db.psql.port")
		user                  = cfg.GetString("storage.db.psql.user")
		password              = cfg.GetString("storage.db.psql.password")
		dbname                = cfg.GetString("storage.db.psql.dbname")
		sslMode               = cfg.GetString("storage.db.psql.sslMode")
		pgSchema              = cfg.GetString("storage.db.psql.schema")
		pgtablesUsersFirebase = cfg.GetString("storage.db.psql.tables.users_firebase")
	)

	portInt, _ := strconv.Atoi(port)

	psqlConn := fmt.Sprintf("host=%s port=%d user=%s password=%s dbname=%s sslMode=%s",
		host, portInt, user, password, dbname, sslMode)

	db, err := sql.Open("postgres", psqlConn)

	if err != nil {
		log.Error().Err(err).Msg("opening connection to postgresql failed")
		return PostgresqlStorage{}, fmt.Errorf("connection to postgresql failed")
	}

	if err = db.Ping(); err != nil {
		log.Error().Err(err).Msg("ping to postgresql failed")
		return PostgresqlStorage{}, fmt.Errorf("connection to postgresql database failed")
	}

	return PostgresqlStorage{
		DbConnection: db,
		Schema:       pgSchema,
		Tables: map[string]string{
			"users_firebase": fmt.Sprintf("%s.%s", pgSchema, pgtablesUsersFirebase),
		},
	}, nil
}

func (s PostgresqlStorage) CreateFirebaseToken(userId string, fcmToken string, devicePlatform string) (err error) {

	if _, err = uuid.Parse(userId); err != nil {
		log.Error().Err(err).Msg("Postgresql Storage CreateFirebaseToken invalid User ID")
		return err
	}
	if _, err = s.DbConnection.Exec(fmt.Sprintf("INSERT INTO %s "+
		"(user_id , fcm_token , device_platform) VALUES($1,$2,$3)", s.Tables["users_firebase"]),
		userId,
		fcmToken,
		devicePlatform); err != nil {
		if strings.Contains(err.Error(), "duplicate key") {
			_ = s.UpdateFirebaseToken(userId, devicePlatform, fcmToken)
		}
	}

	return nil
}
func (s PostgresqlStorage) GetFirebaseToken(userId string) (fcm string, devicePlatform string, err error) {
	if err = s.DbConnection.QueryRow(fmt.Sprintf("SELECT fcm_token , "+
		"device_platform FROM %s WHERE user_id = $1", s.Tables["users_firebase"]), userId).
		Scan(
			&fcm,
			&devicePlatform,
		); err != nil {
		return "", "", err
	}

	return fcm, devicePlatform, nil
}
func (s PostgresqlStorage) UpdateFirebaseToken(userId string, fcmToken string, devicePlatform string) error {
	query := fmt.Sprintf("UPDATE %s SET fcm_token ="+
		" $1 device_platform = $2 WHERE user_id = $3", s.Tables["users_firebase"])

	if _, err := s.DbConnection.Exec(query, fcmToken, devicePlatform, userId); err != nil {
		return err
	}

	return nil
}
